==> g2ipmsg-0.1.tar.gz: g2ipmsg-0.1/src/userdb.c <==
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <glib.h>
#include "callbacks.h"
#include "common.h"

static GList *users=NULL;
GStaticMutex userdb_mutex = G_STATIC_MUTEX_INIT;

static gint 
userdb_find_element(gconstpointer a,gconstpointer b) {
  int rc=-EINVAL;
  userdb_t *user_a,*user_b;

  if ( (!a) || (!b) ) 
    goto out;

  user_a=(userdb_t *)a;
  user_b=(userdb_t *)b;

  if ( (!strncmp(user_a->user,user_b->user,NAME_LEN)) &&
       (!strcmp(user_a->ipaddr,user_b->ipaddr)) ) {
    rc=0;
    goto out; /* found */
  }
  rc=1;

 out:

  return rc; /* Not found */
}
static gint 
userdb_find_by_ipaddr(gconstpointer a,gconstpointer b) {
  int rc=-EINVAL;
  userdb_t *user_a,*user_b;

  if ( (!a) || (!b) ) 
    goto out;

  user_a=(userdb_t *)a;
  user_b=(userdb_t *)b;

  if (!strcmp(user_a->ipaddr,user_b->ipaddr)) 
    return 0;

  rc=-ESRCH;
 out:

  return rc; /* Not found */
}

static void
print_one_user_entry(gpointer data,gpointer user_data) {
  userdb_t *current_user;
  if (!data)
    return;

  current_user=(userdb_t *)data;
  dbg_out("NickName: %s\n",current_user->nickname);
  dbg_out("Group: %s\n",current_user->group);
  dbg_out("User: %s\n",current_user->user);
  dbg_out("Host: %s\n",current_user->host);
  dbg_out("IP Addr: %s\n",current_user->ipaddr);

  return;
}
static int
alloc_user_info(userdb_t **entry){
  userdb_t *new_user;

  if (!entry)
    return -EINVAL;

  new_user=malloc(sizeof(userdb_t));
  if (!new_user)
    return -ENOMEM;

  memset(new_user,0,sizeof(userdb_t));
  *entry=new_user;

  return 0;
}
static int
destroy_user_info_contents(userdb_t *entry){
  if (!entry)
    return -EINVAL;

  if (entry->user)
    free(entry->user);
  if (entry->host)
    free(entry->host);
  if (entry->group)
    free(entry->group);
  if (entry->nickname)
    free(entry->nickname);
  if (entry->ipaddr)
    free(entry->ipaddr);

  memset(entry,0,sizeof(userdb_t));

  return 0;
}

#define strdup_with_check(dest,src,member,err_label)	\
  ((dest)->member)=strdup((src)->member);		\
  if (!((dest)->member))				\
    goto err_label ;                                    
  
static int
copy_user_info(userdb_t *dest,userdb_t *src) {
  int rc=-ENOMEM;

  if ( (!dest) || (!src) )
    return -EINVAL;

  g_assert(!(dest->user));
  g_assert(!(dest->host));
  g_assert(!(dest->group));
  g_assert(!(dest->nickname));
  g_assert(!(dest->ipaddr));

  /*  下記はマクロであることに注意  */
  strdup_with_check(dest,src,user,err_out)
  strdup_with_check(dest,src,host,err_out)
  strdup_with_check(dest,src,group,err_out)
  strdup_with_check(dest,src,nickname,err_out)
  strdup_with_check(dest,src,ipaddr,err_out)
  
  return 0;
 err_out:
  destroy_user_info_contents(dest);
  return rc;
}
int
userdb_search_user(const char *user,const char *ipaddr,const userdb_t **entry_ref){
  int rc=-ESRCH;
  userdb_t srch_user;
  GList *found;

  if ( (!user) || (!ipaddr) || (!entry_ref) )
    return -EINVAL;

  memset(&srch_user,0,sizeof(userdb_t));

  srch_user.user=(char *)user;
  srch_user.ipaddr=(char *)ipaddr;

  g_static_mutex_lock(&userdb_mutex);  
  found=g_list_find_custom (users,&srch_user,userdb_find_element);
  if (found) {
    userdb_t *new_user;
    _assert(found->data);
    rc=-ENOMEM;
    if (alloc_user_info(&new_user)) 
      goto unlock_out;
    rc=0;
    copy_user_info(new_user,found->data);    
    *entry_ref=new_user;
  }
 unlock_out:
  g_static_mutex_unlock(&userdb_mutex);
  return rc;
}
int
userdb_search_user_by_addr(const char *ipaddr,const userdb_t **entry_ref){
  int rc=-ESRCH;
  userdb_t srch_user;
  GList *found;

  if ( (!ipaddr) || (!entry_ref) )
    return -EINVAL;

  memset(&srch_user,0,sizeof(userdb_t));

  srch_user.ipaddr=(char *)ipaddr;

  g_static_mutex_lock(&userdb_mutex);  
  found=g_list_find_custom (users,&srch_user,userdb_find_by_ipaddr);
  if (found) {
    userdb_t *new_user;
    _assert(found->data);

    rc=-ENOMEM;
    if (alloc_user_info(&new_user)) 
      goto unlock_out;
    rc=0;
    copy_user_info(new_user,found->data);
    *entry_ref=new_user;
  }
 unlock_out:
  g_static_mutex_unlock(&userdb_mutex);
  return rc;
}
int
destroy_user_info(userdb_t *entry){
  int rc;

  if (!entry)
    return -EINVAL;

  rc=destroy_user_info_contents(entry);
  if (rc)
    return rc;

  memset(entry,0,sizeof(userdb_t));

  free(entry);

  return 0;
}
void
userdb_print_user_list(void){
  g_list_foreach(users,
		 print_one_user_entry,
		 NULL);  
}
void
userdb_update_group_list(GtkComboBox *widget){
  if (!widget)
    return;

  g_list_foreach(users,
		 update_one_group_entry,
		 widget);  
}

GList *
refer_user_list(void) {
  return users;
}

int 
userdb_add_user(const udp_con_t *con,const msg_data_t *msg){
  int rc=0;
  userdb_t *new_user;
  GList *new_entry;

  if ( (!con) || (!msg) )
    return -EINVAL;

  new_user=g_slice_new(userdb_t);
  if (!new_user)
    return -ENOMEM;

  memset(new_user,0,sizeof(userdb_t));

  rc=-ENOMEM;
  convert_string_internal(refer_user_name_from_msg(msg),(const gchar **)&(new_user->user));
  if (!(new_user->user))
    goto free_slice_out;

  convert_string_internal(refer_host_name_from_msg(msg),(const gchar **)&(new_user->host));
  if (!(new_user->host))
    goto free_user_out;
  
  convert_string_internal(refer_group_name_from_msg(msg),(const gchar **)&(new_user->group));
  if (!(new_user->group))
    goto free_host_out;

  convert_string_internal(refer_nick_name_from_msg(msg),(const gchar **)&(new_user->nickname));
  if (!(new_user->nickname))
    goto free_nickname_out;

  convert_string_internal(inet_ntoa((con->server).sin_addr),(const gchar **)&(new_user->ipaddr));
  if (!(new_user->ipaddr))
    goto free_group_out;

  dbg_out("Add: %s %s %s %s\n",
	  new_user->user,
	  new_user->host,
	  new_user->group,
	  new_user->ipaddr);

  g_static_mutex_lock(&userdb_mutex);
  if (!g_list_find_custom (users,new_user,userdb_find_element))
    users=g_list_append(users,new_user);
  else {
    rc=-EEXIST;
    g_static_mutex_unlock(&userdb_mutex);
    goto free_ipaddr_out; /* Already exist */
  }
  rc=0;
  /*
   * 健全性チェック
   */
  new_entry=g_list_find_custom(users,new_user,userdb_find_element);
  _assert(new_entry);
  _assert(new_entry->data == new_user);
  g_static_mutex_unlock(&userdb_mutex);

  return rc;

 free_ipaddr_out:
  g_free(new_user->ipaddr);
 free_nickname_out:
  g_free(new_user->nickname);
 free_group_out:
  g_free(new_user->group);
 free_host_out:
  g_free(new_user->host);
 free_user_out:
  g_free(new_user->user);
 free_slice_out:
  g_slice_free(userdb_t,new_user);

  return rc;
}

int 
userdb_update_user(const udp_con_t *con,const msg_data_t *msg){
  userdb_t *new_user;
  userdb_t *old_user;
  GList *update_entry;
  GList *new_entry;

  if ( (!con) || (!msg) )
    return -EINVAL;

  new_user=g_slice_new(userdb_t);
  if (!new_user)
    return -ENOMEM;

  memset(new_user,0,sizeof(userdb_t));

  convert_string_internal(refer_user_name_from_msg(msg),(const gchar **)&(new_user->user));
  if (!(new_user->user))
    goto free_slice_out;

  convert_string_internal(refer_host_name_from_msg(msg),(const gchar **)&(new_user->host));
  if (!(new_user->host))
    goto free_user_out;
  
  convert_string_internal(refer_group_name_from_msg(msg),(const gchar **)&(new_user->group));
  if (!(new_user->group))
    goto free_host_out;

  convert_string_internal(refer_nick_name_from_msg(msg),(const gchar **)&(new_user->nickname));
  if (!(new_user->nickname))
    goto free_nickname_out;

  convert_string_internal(inet_ntoa((con->server).sin_addr),(const gchar **)&(new_user->ipaddr));
  if (!(new_user->ipaddr))
    goto free_group_out;

  dbg_out("update: %s %s %s %s\n",
	  new_user->user,
	  new_user->host,
	  new_user->group,
	  new_user->ipaddr);

  g_static_mutex_lock(&userdb_mutex);
  update_entry=g_list_find_custom(users,new_user,userdb_find_element);
  if (!update_entry) {
    g_static_mutex_unlock(&userdb_mutex);
    goto free_ipaddr_out; /* Not exist */
  } else {
    /*
     * 旧来の情報を削除して新しい情報を追加
     */
    old_user=update_entry->data;
    g_free(old_user->ipaddr);
    g_free(old_user->nickname);
    g_free(old_user->group);
    g_free(old_user->host);
    g_free(old_user->user);
    g_slice_free(userdb_t,old_user);
    users=g_list_remove_link(users,update_entry);
    g_list_free(update_entry);
    users=g_list_append(users,new_user);
  }

  /*
   * 健全性チェック
   */
  new_entry=g_list_find_custom(users,new_user,userdb_find_element);
  _assert(new_entry);
  _assert(new_entry->data == new_user);
  g_static_mutex_unlock(&userdb_mutex);
  return 0;

 free_ipaddr_out:
  g_free(new_user->ipaddr);
 free_nickname_out:
  g_free(new_user->nickname);
 free_group_out:
  g_free(new_user->group);
 free_host_out:
  g_free(new_user->host);
 free_user_out:
  g_free(new_user->user);
 free_slice_out:
  g_slice_free(userdb_t,new_user);

  return -ENOMEM;
}
int 
userdb_del_user(const udp_con_t *con,const msg_data_t *msg){
  GList *del_entry;
  userdb_t del_user;
  userdb_t *del_user_p;
  int rc;

  if ( (!con) || (!msg) )
    return -EINVAL;
  

  memset(&del_user,0,sizeof(userdb_t));
  rc=convert_string_internal(refer_user_name_from_msg(msg),(const gchar **)&(del_user.user));
  if (rc)
    return rc;

  rc=convert_string_internal(refer_host_name_from_msg(msg),(const gchar **)&(del_user.host));
  if (rc)
    return rc;

  rc=convert_string_internal(inet_ntoa((con->server).sin_addr),(const gchar **)&(del_user.ipaddr));
  if (rc)
    goto free_user_out;

  dbg_out("del user start\n");
  g_static_mutex_lock(&userdb_mutex);
  del_entry=g_list_find_custom(users,&del_user,userdb_find_element);
  if (!del_entry) {
    dbg_out("No such entry:%s@%s\n",del_user.user,del_user.ipaddr);
    g_static_mutex_unlock(&userdb_mutex);
    goto free_out;
  } 
  del_user_p=del_entry->data;
  g_free(del_user_p->ipaddr);
  g_free(del_user_p->nickname);
  g_free(del_user_p->group);
  g_free(del_user_p->host);
  g_free(del_user_p->user);
  g_slice_free(userdb_t,del_user_p);
  users=g_list_remove_link(users,del_entry);
  g_list_free(del_entry);
  g_static_mutex_unlock(&userdb_mutex);

 free_out:
  dbg_out("del user end\n");
  g_free(del_user.ipaddr);
 free_user_out:
  g_free(del_user.user);
  
  return -ESRCH;
}

int
userdb_init_userdb(void){
  users=NULL;
  return 0;
}

==> g2ipmsg-0.2.tar.gz: g2ipmsg-0.2/src/userdb.c <==
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <glib.h>
#include "callbacks.h"
#include "common.h"

static GList *users=NULL;
GStaticMutex userdb_mutex = G_STATIC_MUTEX_INIT;

static gint 
userdb_find_element(gconstpointer a,gconstpointer b) {
  int rc=-EINVAL;
  userdb_t *user_a,*user_b;

  if ( (!a) || (!b) ) 
    goto out;

  user_a=(userdb_t *)a;
  user_b=(userdb_t *)b;

  if ( (!strncmp(user_a->user,user_b->user,NAME_LEN)) &&
       (!strcmp(user_a->ipaddr,user_b->ipaddr)) ) {
    rc=0;
    goto out; /* found */
  }
  rc=1;

 out:

  return rc; /* Not found */
}
static gint 
userdb_find_by_ipaddr(gconstpointer a,gconstpointer b) {
  int rc=-EINVAL;
  userdb_t *user_a,*user_b;

  if ( (!a) || (!b) ) 
    goto out;

  user_a=(userdb_t *)a;
  user_b=(userdb_t *)b;

  if (!strcmp(user_a->ipaddr,user_b->ipaddr)) 
    return 0;

  rc=-ESRCH;
 out:

  return rc; /* Not found */
}

static void
print_one_user_entry(gpointer data,gpointer user_data) {
  userdb_t *current_user;
  if (!data)
    return;

  current_user=(userdb_t *)data;
  dbg_out("NickName: %s\n",current_user->nickname);
  dbg_out("Group: %s\n",current_user->group);
  dbg_out("User: %s\n",current_user->user);
  dbg_out("Host: %s\n",current_user->host);
  dbg_out("IP Addr: %s\n",current_user->ipaddr);

  return;
}
static int
alloc_user_info(userdb_t **entry){
  userdb_t *new_user;

  if (!entry)
    return -EINVAL;

  new_user=malloc(sizeof(userdb_t));
  if (!new_user)
    return -ENOMEM;

  memset(new_user,0,sizeof(userdb_t));
  *entry=new_user;

  return 0;
}
static int
destroy_user_info_contents(userdb_t *entry){

  if (!entry)
    return -EINVAL;

  if (entry->user)
    free(entry->user);
  if (entry->host)
    free(entry->host);
  if (entry->group)
    free(entry->group);
  if (entry->nickname)
    free(entry->nickname);
  if (entry->ipaddr)
    free(entry->ipaddr);

  memset(entry,0,sizeof(userdb_t));

  return 0;
}

#define strdup_with_check(dest,src,member,err_label)	\
  ((dest)->member)=strdup((src)->member);		\
  if (!((dest)->member))				\
    goto err_label ;                                    
  
static int
copy_user_info(userdb_t *dest,userdb_t *src) {
  int rc=-ENOMEM;

  if ( (!dest) || (!src) )
    return -EINVAL;

  g_assert(!(dest->user));
  g_assert(!(dest->host));
  g_assert(!(dest->group));
  g_assert(!(dest->nickname));
  g_assert(!(dest->ipaddr));

  /*  下記はマクロであることに注意  */
  strdup_with_check(dest,src,user,err_out)
  strdup_with_check(dest,src,host,err_out)
  strdup_with_check(dest,src,group,err_out)
  strdup_with_check(dest,src,nickname,err_out)
  strdup_with_check(dest,src,ipaddr,err_out)
  
  return 0;
 err_out:
  destroy_user_info_contents(dest);
  return rc;
}
static int
fill_user_info_with_message(const char *peer_addr,const msg_data_t *msg,userdb_t *new_user){
  int rc;

  if ( (!new_user) || (!msg) )
    return -EINVAL;

  memset(new_user,0,sizeof(userdb_t));

  rc=-ENOMEM;
  convert_string_internal(refer_user_name_from_msg(msg),(const gchar **)&(new_user->user));
  if (!(new_user->user))
    goto free_slice_out;

  convert_string_internal(refer_host_name_from_msg(msg),(const gchar **)&(new_user->host));
  if (!(new_user->host))
    goto free_user_out;
  
  convert_string_internal(refer_group_name_from_msg(msg),(const gchar **)&(new_user->group));
  if (!(new_user->group))
    goto free_host_out;

  convert_string_internal(refer_nick_name_from_msg(msg),(const gchar **)&(new_user->nickname));
  if (!(new_user->nickname))
    goto free_nickname_out;

  convert_string_internal(peer_addr,(const gchar **)&(new_user->ipaddr));
  if (!(new_user->ipaddr))
    goto free_group_out;

  new_user->cap=msg->command_opts;
  dbg_out("Add: %s %s %s %s %x\n",
	  new_user->user,
	  new_user->host,
	  new_user->group,
	  new_user->ipaddr,
	  (unsigned int)new_user->cap);

  return 0;

 free_ipaddr_out:
  g_free(new_user->ipaddr);
 free_nickname_out:
  g_free(new_user->nickname);
 free_group_out:
  g_free(new_user->group);
 free_host_out:
  g_free(new_user->host);
 free_user_out:
  g_free(new_user->user);
 free_slice_out:
  g_slice_free(userdb_t,new_user);

  return rc;
}
int
userdb_search_user(const char *user,const char *ipaddr,const userdb_t **entry_ref){
  int rc=-ESRCH;
  userdb_t srch_user;
  GList *found;

  if ( (!user) || (!ipaddr) || (!entry_ref) )
    return -EINVAL;

  memset(&srch_user,0,sizeof(userdb_t));

  srch_user.user=(char *)user;
  srch_user.ipaddr=(char *)ipaddr;

  g_static_mutex_lock(&userdb_mutex);  
  found=g_list_find_custom (users,&srch_user,userdb_find_element);
  if (found) {
    userdb_t *new_user;
    _assert(found->data);
    rc=-ENOMEM;
    if (alloc_user_info(&new_user)) 
      goto unlock_out;
    rc=0;
    copy_user_info(new_user,found->data);    
    *entry_ref=new_user;
  }
 unlock_out:
  g_static_mutex_unlock(&userdb_mutex);
  return rc;
}
int
userdb_search_user_by_addr(const char *ipaddr,const userdb_t **entry_ref){
  int rc=-ESRCH;
  userdb_t srch_user;
  GList *found;

  if ( (!ipaddr) || (!entry_ref) )
    return -EINVAL;

  memset(&srch_user,0,sizeof(userdb_t));

  srch_user.ipaddr=(char *)ipaddr;

  g_static_mutex_lock(&userdb_mutex);  
  found=g_list_find_custom (users,&srch_user,userdb_find_by_ipaddr);
  if (found) {
    userdb_t *new_user;
    _assert(found->data);

    rc=-ENOMEM;
    if (alloc_user_info(&new_user)) 
      goto unlock_out;
    rc=0;
    copy_user_info(new_user,found->data);
    *entry_ref=new_user;
  }
 unlock_out:
  g_static_mutex_unlock(&userdb_mutex);
  return rc;
}
int
destroy_user_info(userdb_t *entry){
  int rc;

  if (!entry)
    return -EINVAL;

  rc=destroy_user_info_contents(entry);
  if (rc)
    return rc;

  memset(entry,0,sizeof(userdb_t));

    g_slice_free(userdb_t,entry);

  return 0;
}
void
userdb_print_user_list(void){
  g_list_foreach(users,
		 print_one_user_entry,
		 NULL);  
}
void
userdb_update_group_list(GtkComboBox *widget){
  if (!widget)
    return;

  g_list_foreach(users,
		 update_one_group_entry,
		 widget);  
}

GList *
refer_user_list(void) {
  return users;
}
int 
userdb_add_user(const udp_con_t *con,const msg_data_t *msg){
  int rc=0;
  userdb_t *new_user;
  GList *new_entry;

  if ( (!con) || (!msg) )
    return -EINVAL;

  new_user=g_slice_new(userdb_t);
  if (!new_user)
    return -ENOMEM;

  rc=fill_user_info_with_message(inet_ntoa((con->server).sin_addr),msg,new_user);
  if (rc<0)
    return rc;

  g_static_mutex_lock(&userdb_mutex);
  if (!g_list_find_custom (users,new_user,userdb_find_element))
    users=g_list_append(users,new_user);
  else {
    rc=-EEXIST;
    g_static_mutex_unlock(&userdb_mutex);
    destroy_user_info(new_user);
    return -ENOMEM;
  }
  rc=0;
  /*
   * 健全性チェック
   */
  new_entry=g_list_find_custom(users,new_user,userdb_find_element);
  _assert(new_entry);
  _assert(new_entry->data == new_user);
  g_static_mutex_unlock(&userdb_mutex);

  return rc;
}

int 
userdb_update_user(const udp_con_t *con,const msg_data_t *msg){
  userdb_t *new_user;
  userdb_t *old_user;
  GList *update_entry;
  GList *new_entry;
  int rc;

  if ( (!con) || (!msg) )
    return -EINVAL;

  new_user=g_slice_new(userdb_t);
  if (!new_user)
    return -ENOMEM;

  rc=fill_user_info_with_message(inet_ntoa((con->server).sin_addr),msg,new_user);
  if (rc<0)
    return rc;

  g_static_mutex_lock(&userdb_mutex);
  update_entry=g_list_find_custom(users,new_user,userdb_find_element);
  if (!update_entry) {
    g_static_mutex_unlock(&userdb_mutex);
    destroy_user_info(new_user);
    return -ENOMEM;
  } else {
    /*
     * 旧来の情報を削除して新しい情報を追加
     */
    old_user=update_entry->data;
    g_free(old_user->ipaddr);
    g_free(old_user->nickname);
    g_free(old_user->group);
    g_free(old_user->host);
    g_free(old_user->user);
    g_slice_free(userdb_t,old_user);
    users=g_list_remove_link(users,update_entry);
    g_list_free(update_entry);
    users=g_list_append(users,new_user);
  }

  /*
   * 健全性チェック
   */
  new_entry=g_list_find_custom(users,new_user,userdb_find_element);
  _assert(new_entry);
  _assert(new_entry->data == new_user);
  g_static_mutex_unlock(&userdb_mutex);
  return 0;
}
int 
userdb_del_user(const udp_con_t *con,const msg_data_t *msg){
  GList *del_entry;
  userdb_t del_user;
  userdb_t *del_user_p;
  int rc;

  if ( (!con) || (!msg) )
    return -EINVAL;
  

  memset(&del_user,0,sizeof(userdb_t));
  rc=convert_string_internal(refer_user_name_from_msg(msg),(const gchar **)&(del_user.user));
  if (rc)
    return rc;

  rc=convert_string_internal(refer_host_name_from_msg(msg),(const gchar **)&(del_user.host));
  if (rc)
    return rc;

  rc=convert_string_internal(inet_ntoa((con->server).sin_addr),(const gchar **)&(del_user.ipaddr));
  if (rc)
    goto free_user_out;

  dbg_out("del user start\n");
  g_static_mutex_lock(&userdb_mutex);
  del_entry=g_list_find_custom(users,&del_user,userdb_find_element);
  if (!del_entry) {
    dbg_out("No such entry:%s@%s\n",del_user.user,del_user.ipaddr);
    g_static_mutex_unlock(&userdb_mutex);
    goto free_out;
  } 
  del_user_p=del_entry->data;
  g_free(del_user_p->ipaddr);
  g_free(del_user_p->nickname);
  g_free(del_user_p->group);
  g_free(del_user_p->host);
  g_free(del_user_p->user);
  g_slice_free(userdb_t,del_user_p);
  users=g_list_remove_link(users,del_entry);
  g_list_free(del_entry);
  g_static_mutex_unlock(&userdb_mutex);

 free_out:
  dbg_out("del user end\n");
  g_free(del_user.ipaddr);
 free_user_out:
  g_free(del_user.user);
  
  return -ESRCH;
}

int
userdb_init_userdb(void){
  users=NULL;
  return 0;
}
